(********************************************************************

:Program.     ListSPOs.mod
:Contents.    Shows available SPObjects of superplay.library
:Copyright.   (c) 1994 by Andreas R. Kleinert.
:Copyright.   All rights reserved.
:Language.    Oberon-2
:Translator.  A+L Amiga Oberon Compiler V3.11d
:History.     V1.5 indy 26-Dec-95 first Oberon-2 version
:History.     V5.1 indy 25-Jan-97 Update to C-Version V5.1 (8.8.1996)
:Imports.     SuperPlay.mod
:Imports.     spObjects.mod
*********************************************************************)

MODULE ListSPOs;

IMPORT
  Break,

     d:= Dos,
     e:= Exec,

  args:= Arguments,
    io,
    ol:= OberonLib,
     s:= SYSTEM,

    sp:= SuperPlay,
   spo:= spObjects;


CONST
  (*** Help- and Info- Texts ****)
  VerText    = "\o$VER: ListSPOs(Oberon-2) 5.1 (25.01.1997)";

  EntryText1 = "\[1;32;40mListSPOs(Oberon-2) V5.1 \[0;32;40m\[3;32;40m(FREEWARE)\[0;32;40m\n"
               "(c) 1994-96 by Andreas Ralph Kleinert.\n"
               "Andreas R. Kleinert, Sandstrasse 1, D-57072 Siegen, Germany.\n";
  EntryText2 = 'Lists up available "SPObjects" of "superplay.library".\n';
  EntryText3 = "USAGE : \[0;33;40mListSPOs\[0;31;40m\n";


  (*** Error-Messages for Leave() and KF_Message()     ***)
  SPLibText  = "You need 'superplay.library' V5+ !";


VAR
  verText    : e.STRING;
  argv       : ARRAY 3 OF ARRAY 256 OF CHAR;

  retVal     : LONGINT;
  objList    : e.ListPtr;
  spoNode    : spo.ObjectNodePtr;
  i          : LONGINT;
(*-------------------------------------------------------------------*)

PROCEDURE Leave(endText: ARRAY OF CHAR; result1, result2: LONGINT);

VAR
  procPtr: d.ProcessPtr;
  dummy  : LONGINT;

BEGIN
  procPtr:=s.VAL(d.ProcessPtr, ol.Me);

  IF (endText#"") AND (ol.wbStarted=FALSE) THEN
    io.WriteString(endText); io.WriteLn;
  END;

  IF d.base.lib.version>=36 THEN dummy:= d.SetIoErr(result2);
                            ELSE procPtr.result2:=result2;
  END;

  IF result1= d.ok THEN HALT(d.ok);
  ELSIF result1= d.warn  THEN HALT(d.warn);
  ELSIF result1= d.error THEN HALT(d.error);
  ELSIF result1= d.fail  THEN HALT(d.fail);
  ELSE HALT(d.fail);
  END;
END Leave;
(*-------------------------------------------------------------------*)

BEGIN
  verText:= VerText;

  IF args.NumArgs()>=1 THEN args.GetArg(1, argv[1]); END;
  IF args.NumArgs()>=2 THEN args.GetArg(2, argv[2]); END;
  
  IF (args.NumArgs()>2) OR (argv[1, 0] ="?") OR (argv[2, 0] = "?") THEN
    io.WriteString(EntryText1);
    io.WriteString(EntryText2);
    io.WriteString(EntryText3);
    Leave("", 0, 0);
  END;

  IF (sp.base#NIL) AND (sp.LibVer(sp.base)<5) THEN Leave(SPLibText, d.fail, 100); END;

  io.WriteString("\nList of available SPObjects :");

  objList:= s.ADR(sp.base.spObjectList);
  spoNode:= objList.head;

  WHILE (spoNode#NIL) AND (spoNode#s.ADR(objList.tail)) DO

    io.WriteString("\n\n");

    io.WriteString('\nSPObject : "'); io.WriteString(spoNode.fileName); io.Write('"');
    io.WriteString('\nSPO-Type : ');  io.WriteInt(spoNode.objectType, 1);
    io.WriteString('\nPriority : ');  io.WriteInt(spoNode.node.pri, 1);
    io.WriteString('\nTypeID   : "'); io.WriteString(spoNode.typeID); io.Write('"');
    io.WriteString('\nTypeCode : ');  io.WriteInt(spoNode.typeCode, 1);


    IF spoNode.backgroundReplay=1 THEN io.WriteString("\nKind     : BackGround-Player");
                                  ELSE io.WriteString("\nKind     : Synchron-Player");
    END;

    FOR i:=0 TO spoNode.subTypeNum-1 DO
      io.WriteString('\n\n'); io.WriteInt(i+1, 1); io.WriteString(". SubType     :");
      io.WriteString('\n   SubTypeID   : "');      io.WriteString(spoNode.subTypeID[i]); io.Write('"');
      io.WriteString('\n   SubTypeCode : ');       io.WriteInt(spoNode.subTypeCode[i], 1);
    END;

    spoNode:= spoNode.node.succ;
  END;

  io.WriteString("\n\nNo more SPObjects available.\n\n");
  Leave("", 0, 0);
END ListSPOs.
