(********************************************************************

:Program.     SampleListPlay.mod
:Contents.    SPObject operations
:Contents.    Test SampleList functions.
:Copyright.   (c) 1994-96 by Andreas R. Kleinert.
:Copyright.   Freeware. All rights reserved.
:Language.    Oberon-2
:Translator.  A+L Amiga Oberon Compiler V3.11d
:History.     V2.1 indy 03-Jul-96 first Oberon-2 version
:History.     V5.1 indy 25-Jul-97 Update to C-Verion V5.1 (08.08.1996)
:Imports.     SuperPlay.mod
:Imports.     spObjects.mod
*********************************************************************)

MODULE SampleListPlay;

IMPORT
    ad:= Audio,
   asl:= ASL,
     d:= Dos,
     e:= Exec,
    es:= ExecSupport,
     u:= Utility,

  args:= Arguments,
    io,
    ol:= OberonLib,
   str:= Strings,
     s:= SYSTEM,


    sp:= SuperPlay,
   spo:= spObjects;


CONST
  (*** Help- and Info- Texts ****)
  VerText    = "\o$VER: SamplePlayList 5.1 (22.1.97)";

  EntryText1 = "\[1;32;40mSampleListDemo(Oberon-2) V5.1 \[0;32;40m\[3;32;40m(FREEWARE)\[0;32;40m\n"
               "(c) 1994-96 by Andreas Ralph Kleinert.\n"
               "Andreas R. Kleinert, Sandstrasse 1, D-57072 Siegen, Germany.\n";
  EntryText2 = "Plays Sound Samples and Modules via superplay.library.\n";
  EntryText3 = "USAGE : \[0;33;40mSamplePlayList\[0;31;40m[? | <Sample/ModuleFileName>]\n";


  (*** Error-Messages for Leave() and KF_Message()     ***)
  SPLibText  = 'You need "superplay.library" V5+ !';
  ASLLibText = 'You need "asl.library" V37+ !';


VAR
  verText    : e.STRING;
  argv       : ARRAY 3 OF ARRAY 256 OF CHAR;

  request    : asl.FileRequesterPtr;
  tags       : ARRAY 4 OF u.TagItem;
  nameBuffer : e.STRING;
(*-------------------------------------------------------------------*)

PROCEDURE Leave(endText: ARRAY OF CHAR; result1, result2: LONGINT);

VAR
  procPtr: d.ProcessPtr;
  dummy  : LONGINT;

BEGIN
  procPtr:=s.VAL(d.ProcessPtr, ol.Me);

  IF (endText#"") AND (ol.wbStarted=FALSE) THEN
    io.WriteString(endText); io.WriteLn;
  END;

  IF d.base.lib.version>=36 THEN dummy:= d.SetIoErr(result2);
                            ELSE procPtr.result2:=result2;
  END;

  IF result1= d.ok THEN HALT(d.ok);
  ELSIF result1= d.warn  THEN HALT(d.warn);
  ELSIF result1= d.error THEN HALT(d.error);
  ELSIF result1= d.fail  THEN HALT(d.fail);
  ELSE HALT(d.fail);
  END;
END Leave;
(*-------------------------------------------------------------------*)

PROCEDURE PlaySample(buffer: e.APTR; length, freq, volume: LONGINT): e.STRING;

VAR
 soundPlay, soundLong: BOOLEAN;
 soundLen,
 opres, soundRest    : LONGINT;

 channel             : e.BYTE;
 soundBuf            : e.APTR;
 audioReq            : ad.IOAudioPtr;
 audioPort           : e.MsgPortPtr;
 msg                 : e.MessagePtr;
 retStr              : e.STRING;

BEGIN
 retStr:="";

 audioPort:= es.CreatePort("8SVX.spobject", 0);
 IF audioPort#NIL THEN

   audioReq:= es.CreateExtIO(audioPort, SIZE(ad.IOAudio));
   IF audioReq#NIL THEN

     channel:= 15;

     audioReq.request.message.node.pri:= 10;
     audioReq.data                    := s.ADR(channel);
     audioReq.length                  := 1;
     audioReq.allocKey                := 0;

     opres:= e.OpenDevice("audio.device", 0, audioReq, LONGSET{});
     IF opres=0 THEN

       soundPlay:= TRUE;
       soundBuf := buffer;
       soundRest:= length;

       WHILE soundPlay=TRUE DO

         IF soundRest<= 0FFFFH THEN

           soundLen := soundRest;
           soundLong:= FALSE;
           soundPlay:= FALSE;
         ELSE

           soundLen := 0FFFFH;
           soundLong:= TRUE;

           soundRest:= soundRest-soundLen;
         END;

         audioReq.request.command:= e.write;
         audioReq.request.flags  := SHORTSET{ad.pervol};
         audioReq.data           := soundBuf;
         audioReq.cycles         := 1;
         audioReq.length         := soundLen;
         audioReq.period         := SHORT(3579546 DIV freq);
         audioReq.volume         := SHORT(volume);

         es.BeginIO(audioReq);
         e.OldWaitIO(audioReq);

         soundBuf:= s.VAL(e.APTR, s.VAL(LONGINT, soundBuf)+soundLen);
       END;

       audioReq.request.command := ad.finish;
       audioReq.request.flags   := SHORTSET{};
       audioReq.data            := NIL;
       audioReq.cycles          := 0;
       audioReq.length          := 0;
       audioReq.period          := 0;
       audioReq.volume          := 0;

       e.OldDoIO(audioReq);

       e.CloseDevice(audioReq);

     ELSE retStr:= 'Cant open "audio.device"!';
     END;

     es.DeleteExtIO(audioReq);

   ELSE retStr:= "Not enough memory !";
   END;

   msg:= e.GetMsg(audioPort);
   WHILE msg#NIL DO
     e.ReplyMsg(msg);
     msg:= e.GetMsg(audioPort);
   END;

   es.DeletePort(audioPort);

  ELSE retStr:= "Not enough memory !";
  END;

 RETURN(retStr);
END PlaySample;
(*-------------------------------------------------------------------*)

(* *************************************************** *
 * *                                                 * *
 * * Examine-Function                                * *
 * *                                                 * *
 * *************************************************** *)

PROCEDURE Examine(filename: ARRAY OF CHAR);

VAR
  handle     : e.APTR;
  retVal     : LONGINT;

  sampleList : spo.SampleListPtr;
  entry      : spo.SampleEntryPtr;

  retStr     : e.STRING;

BEGIN
  retVal:= sp.errNoError;

  handle:= sp.AllocHandle(NIL);
  IF handle#NIL THEN   retVal:= sp.InitHandleAsDOS(handle, NIL);
    IF retVal=0 THEN   retVal:= sp.ReadPlayData   (handle, filename);
      IF retVal=0 THEN retVal:= sp.GetSampleList  (handle, s.ADR(sampleList));
        IF retVal=0 THEN

          IF sampleList#NIL THEN
            io.WriteString("\n This File contains ");
            io.WriteInt(sampleList.numEntries, 1);
            io.WriteString(" Entrie(s)\n");
  
            entry:= sampleList.entryList.head;
            WHILE (entry#NIL) AND (entry#s.ADR(sampleList.entryList.tail)) DO
              io.WriteString("\n Entry-Version : "); io.WriteInt(entry.version, 1);
  
              CASE entry.type OF
                | spo.typeSample:
                  io.WriteString("\n Entry-Type    : SAMPLE");
                  io.WriteString("\n SampleBuffer  : 0x"); io.WriteHex(entry.sampleBuffer, 1);
                  io.WriteString("\n SampleSize    : "); io.WriteInt(entry.sampleSize, 1); io.WriteString(" Bytes");
                  io.WriteString("\n SampleBits    : "); io.WriteInt(entry.sampleBits   ,1);
                  io.WriteString("\n SamplesPerSec : "); io.WriteInt(entry.samplesPerSec,1);
                  io.WriteString("\n Volume        : "); io.WriteInt(entry.volume       ,1);
  
                  IF entry.sampleBits = 8 THEN
                    io.WriteString("\n\nPlaying Sample ...");
      
                    retStr:= PlaySample(entry.sampleBuffer, entry.sampleSize,
                                        entry.samplesPerSec, entry.volume);
      
                    IF retStr=""  THEN io.WriteString(" Ready\n");
                                  ELSE io.WriteString("\n");
                                       io.WriteString(retStr);
                                       io.WriteString("\n");
                    END;
                  END;
  
                ELSE io.WriteString("\n Entry-Type    : <Unknown> -> skipped");
              END;
  
              io.WriteString("\n ----------------------------------");
              entry:= entry.node.succ;
            END;
            io.WriteLn;
  
          ELSE
            retVal:= sp.errIllegalAccess;
          END;
        END;
      END;
    END;
    sp.FreeHandle(handle);
  ELSE
    retVal:= sp.errNoHandle;
  END;

 IF retVal#0 THEN Leave(sp.GetErrorString(retVal)^, 0, 0) END;
END Examine;
(*-------------------------------------------------------------------*)

BEGIN
  verText:= VerText;

  IF args.NumArgs()>=1 THEN args.GetArg(1, argv[1]); END;

  IF (args.NumArgs()>1) OR (argv[1, 0] ="?") THEN
    io.WriteString(EntryText1);
    io.WriteString(EntryText2);
    io.WriteString(EntryText3);
    Leave("", 0, 0);
  END;

  IF (sp.base#NIL) AND (sp.LibVer(sp.base)<5) THEN Leave(SPLibText, d.fail, 102); END;
  IF (asl.base#NIL) AND (sp.LibVer(asl.base)<37) THEN Leave(ASLLibText, d.fail, 102); END;


 (* Play *)

 IF args.NumArgs()< 1 THEN

    nameBuffer := "";

    tags[0].tag  := asl.hail;
    tags[0].data := s.ADR("Select Sample/Module to play :");

    tags[1].tag  := asl.okText;
    tags[1].data := s.ADR(" Play ");

    tags[2].tag  := asl.cancelText;
    tags[2].data := s.ADR(" Quit ");

    tags[3].tag  := u.done;
    tags[3].data := NIL;

    request:= asl.AllocAslRequest(asl.fileRequest, NIL);
    IF request#NIL THEN
      IF asl.AslRequest(request, tags)=TRUE THEN

        COPY(request.dir^, nameBuffer);
        IF (nameBuffer[0]# "\o") AND
           (nameBuffer[str.Length(nameBuffer)-1]# ":") AND
           (nameBuffer[str.Length(nameBuffer)-1]# "/") THEN str.Append(nameBuffer, "/");
        END;

        str.Append(nameBuffer, request.file^);
        IF nameBuffer#"" THEN Examine(nameBuffer); END;
      END;
      asl.FreeAslRequest(request);
    END;
  ELSE
    Examine(argv[1]);
  END;

  Leave("", 0, 0);

END SampleListPlay.
